@@ -12,6 +12,7 @@
 #include <QStringBuilder>
 
 #include "controller.h"
+#include "customemojimodel.h"
 #include "roommanager.h"
 
 ActionsHandler::ActionsHandler(QObject *parent)
@@ -61,28 +62,36 @@ void ActionsHandler::setConnection(Connection *connection)
 
 void ActionsHandler::postEdit(const QString &text)
 {
-
     const auto localId = Controller::instance().activeConnection()->userId();
     for (auto it = m_room->messageEvents().crbegin(); it != m_room->messageEvents().crend(); ++it) {
         const auto &evt = **it;
         if (const auto event = eventCast<const RoomMessageEvent>(&evt)) {
             if (event->senderId() == localId && event->hasTextContent()) {
-                static QRegularExpression re("^s/([^/]*)/([^/]*)");
+                static QRegularExpression re("^s/([^/]*)/([^/]*)(/g)?");
                 auto match = re.match(text);
                 if (!match.hasMatch()) {
                     // should not happen but still make sure to send the message normally
                     // just in case.
-                    postMessage(text, QString(), QString(), QString(), QVariantMap());
+                    postMessage(text, QString(), QString(), QString(), QVariantMap(), nullptr);
                 }
                 const QString regex = match.captured(1);
                 const QString replacement = match.captured(2);
+                const QString flags = match.captured(3);
                 QString originalString;
                 if (event->content()) {
                     originalString = static_cast<const Quotient::EventContent::TextContent *>(event->content())->body;
                 } else {
                     originalString = event->plainBody();
                 }
-                m_room->postHtmlMessage(text, originalString.replace(regex, replacement), event->msgtype(), "", event->id());
+                if (flags == "/g") {
+                    m_room->postHtmlMessage(text, originalString.replace(regex, replacement), event->msgtype(), "", event->id());
+                } else {
+                    m_room->postHtmlMessage(text,
+                                            originalString.replace(originalString.indexOf(regex), regex.size(), replacement),
+                                            event->msgtype(),
+                                            "",
+                                            event->id());
+                }
                 return;
             }
         }
@@ -90,20 +99,41 @@ void ActionsHandler::postEdit(const QString &text)
 }
 
 void ActionsHandler::postMessage(const QString &text,
-                                 const QString &attachementPath,
+                                 const QString &attachmentPath,
                                  const QString &replyEventId,
                                  const QString &editEventId,
-                                 const QVariantMap &usernames)
+                                 const QVariantMap &usernames,
+                                 CustomEmojiModel *cem)
 {
     QString rawText = text;
-    QString cleanedText = text;
+    auto stringList = text.split(QStringLiteral("```"));
+    QString cleanedText;
+    const auto count = stringList.count();
+    for (int i = 0; i < count; i++) {
+        if (i % 2 == 0) {
+            if (i + 1 != count) {
+                cleanedText += stringList[i].toHtmlEscaped() + QStringLiteral("```");
+            } else {
+                cleanedText += stringList[i].toHtmlEscaped();
+            }
+        } else {
+            cleanedText += stringList[i] + QStringLiteral("```");
+        }
+    }
+
+    auto preprocess = [cem](const QString &it) -> QString {
+        if (cem == nullptr) {
+            return it;
+        }
+        return cem->preprocessText(it);
+    };
 
     for (auto it = usernames.constBegin(); it != usernames.constEnd(); it++) {
         cleanedText = cleanedText.replace(it.key(), "[" + it.key() + "](https://matrix.to/#/" + it.value().toString() + ")");
     }
 
-    if (attachementPath.length() > 0) {
-        m_room->uploadFile(attachementPath, cleanedText);
+    if (attachmentPath.length() > 0) {
+        m_room->uploadFile(attachmentPath, cleanedText);
     }
 
     if (cleanedText.length() == 0) {
@@ -115,27 +145,32 @@ void ActionsHandler::postMessage(const QString &text,
     // Message commands
     static const QString shrugPrefix = QStringLiteral("/shrug");
     static const QString lennyPrefix = QStringLiteral("/lenny");
-    static const QString plainPrefix = QStringLiteral("/plain "); // TODO
-    static const QString htmlPrefix = QStringLiteral("/html "); // TODO
+    static const QString tableflipPrefix = QStringLiteral("/tableflip");
+    static const QString unflipPrefix = QStringLiteral("/unflip");
+    // static const QString plainPrefix = QStringLiteral("/plain "); // TODO
+    // static const QString htmlPrefix = QStringLiteral("/html "); // TODO
     static const QString rainbowPrefix = QStringLiteral("/rainbow ");
     static const QString rainbowmePrefix = QStringLiteral("/rainbowme ");
+    static const QString spoilerPrefix = QStringLiteral("/spoiler ");
     static const QString mePrefix = QStringLiteral("/me ");
     static const QString noticePrefix = QStringLiteral("/notice ");
 
     // Actions commands
-    static const QString ddgPrefix = QStringLiteral("/ddg "); // TODO
-    static const QString nickPrefix = QStringLiteral("/nick "); // TODO
-    static const QString meroomnickPrefix = QStringLiteral("/myroomnick "); // TODO
-    static const QString roomavatarPrefix = QStringLiteral("/roomavatar "); // TODO
-    static const QString myroomavatarPrefix = QStringLiteral("/myroomavatar "); // TODO
-    static const QString myavatarPrefix = QStringLiteral("/myavatar "); // TODO
+    // static const QString ddgPrefix = QStringLiteral("/ddg "); // TODO
+    // static const QString nickPrefix = QStringLiteral("/nick "); // TODO
+    // static const QString meroomnickPrefix = QStringLiteral("/myroomnick "); // TODO
+    // static const QString roomavatarPrefix = QStringLiteral("/roomavatar "); // TODO
+    // static const QString myroomavatarPrefix = QStringLiteral("/myroomavatar "); // TODO
+    // static const QString myavatarPrefix = QStringLiteral("/myavatar "); // TODO
     static const QString invitePrefix = QStringLiteral("/invite ");
     static const QString joinPrefix = QStringLiteral("/join ");
+    static const QString joinShortPrefix = QStringLiteral("/j ");
     static const QString partPrefix = QStringLiteral("/part");
+    static const QString leavePrefix = QStringLiteral("/leave");
     static const QString ignorePrefix = QStringLiteral("/ignore ");
     static const QString unignorePrefix = QStringLiteral("/unignore ");
-    static const QString queryPrefix = QStringLiteral("/query "); // TODO
-    static const QString msgPrefix = QStringLiteral("/msg "); // TODO
+    // static const QString queryPrefix = QStringLiteral("/query "); // TODO
+    // static const QString msgPrefix = QStringLiteral("/msg "); // TODO
     static const QString reactPrefix = QStringLiteral("/react ");
 
     // Admin commands
@@ -157,13 +192,33 @@ void ActionsHandler::postMessage(const QString &text,
         return;
     }
 
+    if (cleanedText.indexOf(tableflipPrefix) == 0) {
+        cleanedText = QStringLiteral("(╯°□°）╯︵ ┻━┻") % cleanedText.remove(0, tableflipPrefix.length());
+        m_room->postHtmlMessage(cleanedText, cleanedText, messageEventType, replyEventId, editEventId);
+        return;
+    }
+
+    if (cleanedText.indexOf(unflipPrefix) == 0) {
+        cleanedText = QStringLiteral("┬──┬ ノ( ゜-゜ノ)") % cleanedText.remove(0, unflipPrefix.length());
+        m_room->postHtmlMessage(cleanedText, cleanedText, messageEventType, replyEventId, editEventId);
+        return;
+    }
+
     if (cleanedText.indexOf(rainbowPrefix) == 0) {
         cleanedText = cleanedText.remove(0, rainbowPrefix.length());
         QString rainbowText;
         for (int i = 0; i < cleanedText.length(); i++) {
             rainbowText = rainbowText % QStringLiteral("<font color='") % rainbowColors.at(i % rainbowColors.length()) % "'>" % cleanedText.at(i) % "</font>";
         }
-        m_room->postHtmlMessage(cleanedText, rainbowText, RoomMessageEvent::MsgType::Notice, replyEventId, editEventId);
+        m_room->postHtmlMessage(cleanedText, preprocess(rainbowText), RoomMessageEvent::MsgType::Notice, replyEventId, editEventId);
+        return;
+    }
+
+    if (cleanedText.indexOf(spoilerPrefix) == 0) {
+        cleanedText = cleanedText.remove(0, spoilerPrefix.length());
+        const QStringList splittedText = rawText.split(" ");
+        QString spoilerHtml = QStringLiteral("<span data-mx-spoiler>") % preprocess(cleanedText) % QStringLiteral("</span>");
+        m_room->postHtmlMessage(cleanedText, spoilerHtml, RoomMessageEvent::MsgType::Notice, replyEventId, editEventId);
         return;
     }
 
@@ -173,12 +228,16 @@ void ActionsHandler::postMessage(const QString &text,
         for (int i = 0; i < cleanedText.length(); i++) {
             rainbowText = rainbowText % QStringLiteral("<font color='") % rainbowColors.at(i % rainbowColors.length()) % "'>" % cleanedText.at(i) % "</font>";
         }
-        m_room->postHtmlMessage(cleanedText, rainbowText, messageEventType, replyEventId, editEventId);
+        m_room->postHtmlMessage(cleanedText, preprocess(rainbowText), messageEventType, replyEventId, editEventId);
         return;
     }
 
-    if (rawText.indexOf(joinPrefix) == 0) {
-        rawText = rawText.remove(0, joinPrefix.length());
+    if (rawText.indexOf(joinPrefix) == 0 || rawText.indexOf(joinShortPrefix) == 0) {
+        if (rawText.indexOf(joinPrefix) == 0) {
+            rawText = rawText.remove(0, joinPrefix.length());
+        } else {
+            rawText = rawText.remove(0, joinShortPrefix.length());
+        }
         const QStringList splittedText = rawText.split(" ");
         if (text.count() == 0) {
             Q_EMIT showMessage(MessageType::Error, i18n("Invalid command"));
@@ -207,8 +266,12 @@ void ActionsHandler::postMessage(const QString &text,
         return;
     }
 
-    if (rawText.indexOf(partPrefix) == 0) {
-        rawText = rawText.remove(0, partPrefix.length());
+    if (rawText.indexOf(partPrefix) == 0 || rawText.indexOf(leavePrefix) == 0) {
+        if (rawText.indexOf(partPrefix) == 0) {
+            rawText = rawText.remove(0, partPrefix.length());
+        } else {
+            rawText = rawText.remove(0, leavePrefix.length());
+        }
         const QStringList splittedText = rawText.split(" ");
         if (splittedText.count() == 0 || splittedText[0].isEmpty()) {
             // leave current room
@@ -256,10 +319,18 @@ void ActionsHandler::postMessage(const QString &text,
     }
 
     if (rawText.indexOf(reactPrefix) == 0) {
+        rawText = rawText.remove(0, reactPrefix.length());
         if (replyEventId.isEmpty()) {
+            for (auto it = m_room->messageEvents().crbegin(); it != m_room->messageEvents().crend(); ++it) {
+                const auto &evt = **it;
+                if (const auto event = eventCast<const RoomMessageEvent>(&evt)) {
+                    m_room->toggleReaction(event->id(), rawText);
+                    return;
+                }
+            }
+            Q_EMIT showMessage(MessageType::Error, i18n("Couldn't find a message to react to"));
             return;
         }
-        rawText = rawText.remove(0, reactPrefix.length());
         m_room->toggleReaction(replyEventId, rawText);
         return;
     }
@@ -272,5 +343,5 @@ void ActionsHandler::postMessage(const QString &text,
         cleanedText = cleanedText.remove(0, noticePrefix.length());
         messageEventType = RoomMessageEvent::MsgType::Notice;
     }
-    m_room->postMessage(rawText, cleanedText, messageEventType, replyEventId, editEventId);
+    m_room->postMessage(rawText, preprocess(m_room->preprocessText(cleanedText)), messageEventType, replyEventId, editEventId);
 }
