@@ -10,14 +10,13 @@
 #include <KConfigGroup>
 #include <KLocalizedString>
 #include <KWindowConfig>
+#ifdef HAVE_WINDOWSYSTEM
+#include <KWindowEffects>
+#include <KWindowSystem>
+#endif
 
 #include <QAuthenticator>
 #include <QClipboard>
-#include <QCloseEvent>
-#include <QDebug>
-#include <QDesktopServices>
-#include <QDir>
-#include <QElapsedTimer>
 #include <QFile>
 #include <QFileInfo>
 #include <QGuiApplication>
@@ -25,6 +24,7 @@
 #include <QNetworkConfigurationManager>
 #include <QNetworkReply>
 #include <QPixmap>
+#include <QQuickItem>
 #include <QQuickWindow>
 #include <QStandardPaths>
 #include <QStringBuilder>
@@ -34,25 +34,26 @@
 
 #include <signal.h>
 
+#include "accountregistry.h"
 #include "csapi/account-data.h"
 #include "csapi/content-repo.h"
 #include "csapi/joining.h"
 #include "csapi/logout.h"
 #include "csapi/profile.h"
-#include "csapi/registration.h"
-#include "csapi/wellknown.h"
 #include "events/eventcontent.h"
-#include "events/roommessageevent.h"
 #include "neochatconfig.h"
 #include "neochatroom.h"
 #include "neochatuser.h"
+#include "roommanager.h"
 #include "settings.h"
 #include "utils.h"
-#include "roommanager.h"
+
 #include <KStandardShortcut>
 
-#ifndef Q_OS_ANDROID
+#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
 #include "trayicon.h"
+#elif !defined(Q_OS_ANDROID)
+#include "trayicon_sni.h"
 #endif
 
 using namespace Quotient;
@@ -71,7 +72,7 @@ Controller::Controller(QObject *parent)
         connect(trayIcon, &TrayIcon::showWindow, this, &Controller::showWindow);
         QGuiApplication::setQuitOnLastWindowClosed(false);
     }
-    connect(NeoChatConfig::self(), &NeoChatConfig::SystemTrayChanged, this, [=]() {
+    connect(NeoChatConfig::self(), &NeoChatConfig::SystemTrayChanged, this, [this, trayIcon]() {
         if (NeoChatConfig::self()->systemTray()) {
             trayIcon->show();
             connect(trayIcon, &TrayIcon::showWindow, this, &Controller::showWindow);
@@ -83,7 +84,7 @@ Controller::Controller(QObject *parent)
     });
 #endif
 
-    QTimer::singleShot(0, this, [=] {
+    QTimer::singleShot(0, this, [this] {
         invokeLogin();
     });
 
@@ -115,13 +116,12 @@ Controller::Controller(QObject *parent)
     }
 #endif
 
-    connect(m_mgr, &QNetworkConfigurationManager::onlineStateChanged,
-            this, &Controller::isOnlineChanged);
+    connect(m_mgr, &QNetworkConfigurationManager::onlineStateChanged, this, &Controller::isOnlineChanged);
 }
 
 Controller::~Controller()
 {
-    for (auto c : qAsConst(m_connections)) {
+    for (auto c : AccountRegistry::instance().accounts()) {
         c->saveState();
     }
 }
@@ -147,15 +147,14 @@ void Controller::loginWithAccessToken(const QString &serverAddr, const QString &
 
     QUrl serverUrl(serverAddr);
 
-    auto conn = new Connection(this);
+    auto conn = new Connection();
     if (serverUrl.isValid()) {
         conn->setHomeserver(serverUrl);
     }
 
-    connect(conn, &Connection::connected, this, [=] {
+    connect(conn, &Connection::connected, this, [this, conn, deviceName] {
         AccountSettings account(conn->userId());
         account.setKeepLoggedIn(true);
-        account.clearAccessToken(); // Drop the legacy - just in case
         account.setHomeserver(conn->homeserver());
         account.setDeviceId(conn->deviceId());
         account.setDeviceName(deviceName);
@@ -166,10 +165,10 @@ void Controller::loginWithAccessToken(const QString &serverAddr, const QString &
         addConnection(conn);
         setActiveConnection(conn);
     });
-    connect(conn, &Connection::networkError, this, [=](QString error, const QString &, int, int) {
+    connect(conn, &Connection::networkError, this, [this](QString error, const QString &, int, int) {
         Q_EMIT errorOccured(i18n("Network Error: %1", error));
     });
-    conn->connectWithToken(user, token, deviceName);
+    conn->assumeIdentity(user, token, deviceName);
 }
 
 void Controller::logout(Connection *conn, bool serverSideLogout)
@@ -190,32 +189,28 @@ void Controller::logout(Connection *conn, bool serverSideLogout)
     job.start();
     loop.exec();
 
-    conn->stopSync();
-    Q_EMIT conn->stateChanged();
-    Q_EMIT conn->loggedOut();
-    if (conn == activeConnection() && !m_connections.isEmpty()) {
-        setActiveConnection(m_connections[0]);
+    if (conn == activeConnection() && AccountRegistry::instance().count() > 1) {
+        setActiveConnection(AccountRegistry::instance().accounts()[0]);
     } else {
         setActiveConnection(nullptr);
     }
     if (!serverSideLogout) {
         return;
     }
-    auto logoutJob = conn->callApi<LogoutJob>();
-    connect(logoutJob, &LogoutJob::failure, this, [=] {
-        Q_EMIT errorOccured(i18n("Server-side Logout Failed: %1", logoutJob->errorString()));
-    });
+    conn->logout();
 }
 
 void Controller::addConnection(Connection *c)
 {
     Q_ASSERT_X(c, __FUNCTION__, "Attempt to add a null connection");
 
-    m_connections += c;
+#ifndef QUOTIENT_07
+    AccountRegistry::instance().add(c);
+#endif
 
     c->setLazyLoading(true);
 
-    connect(c, &Connection::syncDone, this, [=] {
+    connect(c, &Connection::syncDone, this, [this, c] {
         setBusy(false);
 
         Q_EMIT syncDone();
@@ -223,11 +218,11 @@ void Controller::addConnection(Connection *c)
         c->sync(30000);
         c->saveState();
     });
-    connect(c, &Connection::loggedOut, this, [=] {
+    connect(c, &Connection::loggedOut, this, [this, c] {
         dropConnection(c);
     });
 
-    connect(c, &Connection::requestFailed, this, [=](BaseJob *job) {
+    connect(c, &Connection::requestFailed, this, [this](BaseJob *job) {
         if (job->error() == BaseJob::UserConsentRequiredError) {
             Q_EMIT userConsentRequired(job->errorUrl());
         }
@@ -244,11 +239,16 @@ void Controller::addConnection(Connection *c)
 void Controller::dropConnection(Connection *c)
 {
     Q_ASSERT_X(c, __FUNCTION__, "Attempt to drop a null connection");
-    m_connections.removeOne(c);
+
+#ifndef QUOTIENT_07
+    AccountRegistry::instance().drop(c);
+#endif
 
     Q_EMIT connectionDropped(c);
     Q_EMIT accountCountChanged();
+#ifndef QUOTIENT_07
     c->deleteLater();
+#endif
 }
 
 void Controller::invokeLogin()
@@ -264,29 +264,29 @@ void Controller::invokeLogin()
         if (!account.homeserver().isEmpty()) {
             auto accessToken = loadAccessTokenFromKeyChain(account);
 
-            auto connection = new Connection(account.homeserver(), this);
-            connect(connection, &Connection::connected, this, [=] {
+            auto connection = new Connection(account.homeserver());
+            connect(connection, &Connection::connected, this, [this, connection, id] {
                 connection->loadState();
                 addConnection(connection);
                 if (connection->userId() == id) {
                     setActiveConnection(connection);
-                    Q_EMIT initiated();
+                    connectSingleShot(connection, &Connection::syncDone, this, &Controller::initiated);
                 }
             });
-            connect(connection, &Connection::loginError, this, [=](const QString &error, const QString &) {
+            connect(connection, &Connection::loginError, this, [this, connection](const QString &error, const QString &) {
                 if (error == "Unrecognised access token") {
                     Q_EMIT errorOccured(i18n("Login Failed: Access Token invalid or revoked"));
                     logout(connection, false);
                 } else {
-                    Q_EMIT errorOccured(i18n("Login Failed", error));
+                    Q_EMIT errorOccured(i18n("Login Failed: %1", error));
                     logout(connection, true);
                 }
                 Q_EMIT initiated();
             });
-            connect(connection, &Connection::networkError, this, [=](const QString &error, const QString &, int, int) {
+            connect(connection, &Connection::networkError, this, [this](const QString &error, const QString &, int, int) {
                 Q_EMIT errorOccured(i18n("Network Error: %1", error));
             });
-            connection->connectWithToken(account.userId(), accessToken, account.deviceId());
+            connection->assumeIdentity(account.userId(), accessToken, account.deviceId());
         }
     }
     if (accounts.isEmpty()) {
@@ -311,23 +311,30 @@ QByteArray Controller::loadAccessTokenFromFile(const AccountSettings &account)
 
 QByteArray Controller::loadAccessTokenFromKeyChain(const AccountSettings &account)
 {
-    qDebug() << "Read the access token from the keychain for " << account.userId();
-    QKeychain::ReadPasswordJob job(qAppName());
-    job.setAutoDelete(false);
-    job.setKey(account.userId());
-    QEventLoop loop;
-    QKeychain::ReadPasswordJob::connect(&job, &QKeychain::Job::finished, &loop, &QEventLoop::quit);
-    job.start();
-    loop.exec();
+    QKeychain::Error error;
+    QString errorString;
+    do {
+        qDebug() << "Reading access token from the keychain for" << account.userId();
+        QKeychain::ReadPasswordJob job(qAppName());
+        job.setAutoDelete(false);
+        job.setKey(account.userId());
+        QEventLoop loop;
+        QKeychain::ReadPasswordJob::connect(&job, &QKeychain::Job::finished, &loop, &QEventLoop::quit);
+        job.start();
+        loop.exec();
 
-    if (job.error() == QKeychain::Error::NoError) {
-        return job.binaryData();
-    }
+        if (job.error() == QKeychain::Error::NoError) {
+            return job.binaryData();
+        }
+        Q_EMIT globalErrorOccured(i18n("Unable to read access token"), i18n("Please make sure that the keychain is opened."));
+        error = job.error();
+        errorString = job.errorString();
+    } while (error == QKeychain::Error::OtherError);
 
-    qWarning() << "Could not read the access token from the keychain: " << qPrintable(job.errorString());
+    qWarning() << "Could not read the access token from the keychain:" << errorString;
     // no access token from the keychain, try token file
     auto accessToken = loadAccessTokenFromFile(account);
-    if (job.error() == QKeychain::Error::EntryNotFound) {
+    if (error == QKeychain::Error::EntryNotFound) {
         if (!accessToken.isEmpty()) {
             qDebug() << "Migrating the access token from file to the keychain for " << account.userId();
             bool removed = false;
@@ -386,7 +393,7 @@ void Controller::playAudio(const QUrl &localFile)
     auto player = new QMediaPlayer;
     player->setMedia(localFile);
     player->play();
-    connect(player, &QMediaPlayer::stateChanged, [=] {
+    connect(player, &QMediaPlayer::stateChanged, [player] {
         player->deleteLater();
     });
 }
@@ -395,7 +402,7 @@ void Controller::changeAvatar(Connection *conn, const QUrl &localFile)
 {
     auto job = conn->uploadFile(localFile.toLocalFile());
 #ifdef QUOTIENT_07
-    if(isJobPending(job)) {
+    if (isJobPending(job)) {
 #else
     if (isJobRunning(job)) {
 #endif
@@ -467,7 +474,11 @@ bool Controller::setAvatar(Connection *connection, const QUrl &avatarSource)
     User *localUser = connection->user();
     QString decoded = avatarSource.path();
     if (decoded.isEmpty()) {
-        connection->callApi<SetAvatarUrlJob>(localUser->id(), "");
+#ifdef QUOTIENT_07
+        connection->callApi<SetAvatarUrlJob>(localUser->id(), avatarSource);
+#else
+        connection->callApi<SetAvatarUrlJob>(localUser->id(), QString());
+#endif
         return true;
     }
     if (QImageReader(decoded).read().isNull()) {
@@ -478,7 +489,11 @@ bool Controller::setAvatar(Connection *connection, const QUrl &avatarSource)
 }
 
 NeochatChangePasswordJob::NeochatChangePasswordJob(const QString &newPassword, bool logoutDevices, const Omittable<QJsonObject> &auth)
-    : BaseJob(HttpVerb::Post, QStringLiteral("ChangePasswordJob"), QStringLiteral("/_matrix/client/r0") % "/account/password")
+#ifdef QUOTIENT_07
+    : BaseJob(HttpVerb::Post, QStringLiteral("ChangePasswordJob"), "/_matrix/client/r0/account/password")
+#else
+    : BaseJob(HttpVerb::Post, QStringLiteral("ChangePasswordJob"), QStringLiteral("/_matrix/client/r0/account/password"))
+#endif
 {
     QJsonObject _data;
     addParam<>(_data, QStringLiteral("new_password"), newPassword);
@@ -487,14 +502,9 @@ NeochatChangePasswordJob::NeochatChangePasswordJob(const QString &newPassword, b
     setRequestData(_data);
 }
 
-QVector<Connection *> Controller::connections() const
-{
-    return m_connections;
-}
-
 int Controller::accountCount() const
 {
-    return m_connections.count();
+    return AccountRegistry::instance().count();
 }
 
 bool Controller::quitOnLastWindowClosed()
@@ -557,7 +567,11 @@ void Controller::saveWindowGeometry(QQuickWindow *window)
 }
 
 NeochatDeleteDeviceJob::NeochatDeleteDeviceJob(const QString &deviceId, const Omittable<QJsonObject> &auth)
+#ifdef QUOTIENT_07
+    : Quotient::BaseJob(HttpVerb::Delete, QStringLiteral("DeleteDeviceJob"), QStringLiteral("/_matrix/client/r0/devices/%1").arg(deviceId).toLatin1())
+#else
     : Quotient::BaseJob(HttpVerb::Delete, QStringLiteral("DeleteDeviceJob"), QStringLiteral("/_matrix/client/r0/devices/%1").arg(deviceId))
+#endif
 {
     QJsonObject _data;
     addParam<IfNotEmpty>(_data, QStringLiteral("auth"), auth);
@@ -567,7 +581,7 @@ NeochatDeleteDeviceJob::NeochatDeleteDeviceJob(const QString &deviceId, const Om
 void Controller::createRoom(const QString &name, const QString &topic)
 {
     auto createRoomJob = m_connection->createRoom(Connection::PublishRoom, "", name, topic, QStringList());
-    Quotient::CreateRoomJob::connect(createRoomJob, &CreateRoomJob::failure, [=] {
+    Quotient::CreateRoomJob::connect(createRoomJob, &CreateRoomJob::failure, [this, createRoomJob] {
         Q_EMIT errorOccured(i18n("Room creation failed: \"%1\"", createRoomJob->errorString()));
     });
 }
@@ -577,6 +591,7 @@ bool Controller::isOnline() const
     return m_mgr->isOnline();
 }
 
+// TODO: Remove in favor of RoomManager::joinRoom
 void Controller::joinRoom(const QString &alias)
 {
     if (!alias.contains(":")) {
@@ -585,14 +600,7 @@ void Controller::joinRoom(const QString &alias)
     }
 
     const auto knownServer = alias.mid(alias.indexOf(":") + 1);
-    auto joinRoomJob = m_connection->joinRoom(alias, QStringList{knownServer});
-
-    connect(joinRoomJob, &JoinRoomJob::failure, [=] {
-        Q_EMIT errorOccured(i18n("Server error when joining the room \"%1\": %2", joinRoomJob->errorString()));
-    });
-    connect(joinRoomJob, &JoinRoomJob::success, [this, joinRoomJob] {
-        Q_EMIT errorOccured(joinRoomJob->roomId());
-    });
+    RoomManager::instance().joinRoom(m_connection, alias, QStringList{knownServer});
 }
 
 void Controller::openOrCreateDirectChat(NeoChatUser *user)
@@ -608,10 +616,43 @@ void Controller::openOrCreateDirectChat(NeoChatUser *user)
 
 QString Controller::formatByteSize(double size, int precision) const
 {
-    return KFormat().formatByteSize(size, precision);
+    return QLocale().formattedDataSize(size, precision);
 }
 
 QString Controller::formatDuration(quint64 msecs, KFormat::DurationFormatOptions options) const
 {
     return KFormat().formatDuration(msecs, options);
 }
+
+void Controller::setBlur(QQuickItem *item, bool blur)
+{
+#ifdef HAVE_WINDOWSYSTEM
+    auto setWindows = [item, blur]() {
+        auto reg = QRect(QPoint(0, 0), item->window()->size());
+        KWindowEffects::enableBackgroundContrast(item->window(), blur, 1, 1, 1, reg);
+        KWindowEffects::enableBlurBehind(item->window(), blur, reg);
+    };
+
+    disconnect(item->window(), &QQuickWindow::heightChanged, this, nullptr);
+    disconnect(item->window(), &QQuickWindow::widthChanged, this, nullptr);
+    connect(item->window(), &QQuickWindow::heightChanged, this, setWindows);
+    connect(item->window(), &QQuickWindow::widthChanged, this, setWindows);
+    setWindows();
+#endif
+}
+
+void Controller::raiseWindow(QWindow *window)
+{
+#ifdef HAVE_WINDOWSYSTEM
+    KWindowSystem::activateWindow(window->winId());
+#endif
+}
+
+bool Controller::hasWindowSystem() const
+{
+#ifdef HAVE_WINDOWSYSTEM
+    return true;
+#else
+    return false;
+#endif
+}
