@@ -5,6 +5,7 @@
 
 #include <QMediaPlayer>
 #include <QObject>
+#include <QQuickItem>
 
 #include <KAboutData>
 #include <KFormat>
@@ -33,13 +34,12 @@ class Controller : public QObject
     Q_PROPERTY(bool busy READ busy WRITE setBusy NOTIFY busyChanged)
     Q_PROPERTY(KAboutData aboutData READ aboutData WRITE setAboutData NOTIFY aboutDataChanged)
     Q_PROPERTY(bool supportSystemTray READ supportSystemTray CONSTANT)
+    Q_PROPERTY(bool hasWindowSystem READ hasWindowSystem CONSTANT)
     Q_PROPERTY(bool isOnline READ isOnline NOTIFY isOnlineChanged)
 
 public:
     static Controller &instance();
 
-    [[nodiscard]] QVector<Connection *> connections() const;
-
     void setActiveConnection(Connection *connection);
     [[nodiscard]] Connection *activeConnection() const;
 
@@ -87,11 +87,14 @@ public:
     Q_INVOKABLE QString formatByteSize(double size, int precision = 1) const;
 
     Q_INVOKABLE void openOrCreateDirectChat(NeoChatUser *user);
+
+    Q_INVOKABLE void setBlur(QQuickItem *item, bool blur);
+    Q_INVOKABLE void raiseWindow(QWindow *window);
+
 private:
     explicit Controller(QObject *parent = nullptr);
     ~Controller() override;
 
-    QVector<Connection *> m_connections;
     QPointer<Connection> m_connection;
     bool m_busy = false;
 
@@ -102,6 +105,7 @@ private:
     void saveSettings() const;
 
     KAboutData m_aboutData;
+    bool hasWindowSystem() const;
 
 private Q_SLOTS:
     void invokeLogin();
