@@ -2,15 +2,16 @@
 // SPDX-License-Identifier: GPL-3.0-only
 
 #include <QCommandLineParser>
+#include <QDebug>
 #include <QFontDatabase>
 #include <QGuiApplication>
+#include <QIcon>
 #include <QNetworkProxy>
 #include <QNetworkProxyFactory>
 #include <QQmlApplicationEngine>
 #include <QQmlContext>
 #include <QQuickStyle>
 #include <QQuickWindow>
-#include <QDebug>
 
 #ifdef Q_OS_ANDROID
 #include <QGuiApplication>
@@ -21,6 +22,7 @@
 #include <KAboutData>
 #ifdef HAVE_KDBUSADDONS
 #include <KDBusService>
+#include <KWindowSystem>
 #endif
 #include <KLocalizedContext>
 #include <KLocalizedString>
@@ -28,8 +30,9 @@
 
 #include "neochat-version.h"
 
-#include "accountlistmodel.h"
+#include "accountregistry.h"
 #include "actionshandler.h"
+#include "blurhashimageprovider.h"
 #include "chatboxhelper.h"
 #include "chatdocumenthandler.h"
 #include "clipboard.h"
@@ -37,9 +40,11 @@
 #include "controller.h"
 #include "csapi/joining.h"
 #include "csapi/leaving.h"
+#include "customemojimodel.h"
 #include "devicesmodel.h"
 #include "emojimodel.h"
 #include "filetypesingleton.h"
+#include "joinrulesevent.h"
 #include "login.h"
 #include "matriximageprovider.h"
 #include "messageeventmodel.h"
@@ -49,15 +54,32 @@
 #include "neochatuser.h"
 #include "notificationsmanager.h"
 #include "publicroomlistmodel.h"
-#include <room.h>
 #include "roomlistmodel.h"
 #include "roommanager.h"
 #include "sortfilterroomlistmodel.h"
+#include "spellcheckhighlighter.h"
 #include "userdirectorylistmodel.h"
 #include "userlistmodel.h"
+#include "webshortcutmodel.h"
+#include <room.h>
+#ifdef HAVE_COLORSCHEME
+#include "colorschemer.h"
+#endif
 
 using namespace Quotient;
 
+#ifdef HAVE_KDBUSADDONS
+static void raiseWindow(QWindow *window)
+{
+    if (KWindowSystem::isPlatformWayland()) {
+        KWindowSystem::setCurrentXdgActivationToken(qEnvironmentVariable("XDG_ACTIVATION_TOKEN"));
+        KWindowSystem::activateWindow(window->winId());
+    } else {
+        window->raise();
+    }
+}
+#endif
+
 #ifdef Q_OS_ANDROID
 Q_DECL_EXPORT
 #endif
@@ -69,8 +91,9 @@ int main(int argc, char *argv[])
 
 #ifdef Q_OS_ANDROID
     QGuiApplication app(argc, argv);
-    QQuickStyle::setStyle(QStringLiteral("Material"));
+    QQuickStyle::setStyle(QStringLiteral("org.kde.breeze"));
 #else
+    QIcon::setFallbackThemeName("breeze");
     QApplication app(argc, argv);
     // Default to org.kde.desktop style unless the user forces another style
     if (qEnvironmentVariableIsEmpty("QT_QUICK_CONTROLS_STYLE")) {
@@ -97,28 +120,34 @@ int main(int argc, char *argv[])
                      QStringLiteral(NEOCHAT_VERSION_STRING),
                      i18n("Matrix client"),
                      KAboutLicense::GPL_V3,
-                     i18n("© 2018-2020 Black Hat, 2020 KDE Community"));
+                     i18n("© 2018-2020 Black Hat, 2020-2021 KDE Community"));
     about.addAuthor(i18n("Black Hat"), QString(), QStringLiteral("bhat@encom.eu.org"));
     about.addAuthor(i18n("Carl Schwan"), QString(), QStringLiteral("carl@carlschwan.eu"));
     about.addAuthor(i18n("Tobias Fella"), QString(), QStringLiteral("fella@posteo.de"));
     about.setOrganizationDomain("kde.org");
 
+    about.addComponent(QStringLiteral("libQuotient"),
+                       i18n("A Qt5 library to write cross-platform clients for Matrix"),
+                       QString(),
+                       QStringLiteral("https://github.com/quotient-im/libquotient"),
+                       KAboutLicense::LGPL_V2_1);
+
     KAboutData::setApplicationData(about);
     QGuiApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("org.kde.neochat")));
 
 #ifdef HAVE_KDBUSADDONS
     KDBusService service(KDBusService::Unique);
-    service.connect(&service,
-                    &KDBusService::activateRequested,
-                    &RoomManager::instance(),
-                    [](const QStringList &arguments, const QString &workingDirectory) {
-                        Q_UNUSED(workingDirectory);
-                        auto args = arguments;
-                        args.removeFirst();
-                        for (const auto &arg : args) {
-                            RoomManager::instance().openResource(arg);
-                        }
-                    });
+    service.connect(&service, &KDBusService::activateRequested, &RoomManager::instance(), [](const QStringList &arguments, const QString &workingDirectory) {
+        Q_UNUSED(workingDirectory);
+        if (arguments.isEmpty()) {
+            return;
+        }
+        auto args = arguments;
+        args.removeFirst();
+        for (const auto &arg : args) {
+            RoomManager::instance().openResource(arg);
+        }
+    });
 #endif
 
 #ifdef NEOCHAT_FLATPAK
@@ -134,36 +163,39 @@ int main(int argc, char *argv[])
     Login *login = new Login();
     ChatBoxHelper chatBoxHelper;
 
+#ifdef HAVE_COLORSCHEME
+    ColorSchemer colorScheme;
+    qmlRegisterSingletonInstance<ColorSchemer>("org.kde.neochat", 1, 0, "ColorSchemer", &colorScheme);
+    if (!config->colorScheme().isEmpty()) {
+        colorScheme.apply(config->colorScheme());
+    }
+#endif
+
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Controller", &Controller::instance());
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Clipboard", &clipboard);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Config", config);
-    qmlRegisterSingletonInstance<RoomManager>("org.kde.neochat", 1, 0, "RoomManager", &RoomManager::instance());
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "RoomManager", &RoomManager::instance());
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "FileType", &fileTypeSingleton);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "LoginHelper", login);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "ChatBoxHelper", &chatBoxHelper);
-    qmlRegisterType<AccountListModel>("org.kde.neochat", 1, 0, "AccountListModel");
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "EmojiModel", new EmojiModel(&app));
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "CommandModel", new CommandModel(&app));
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "AccountRegistry", &Quotient::AccountRegistry::instance());
     qmlRegisterType<ActionsHandler>("org.kde.neochat", 1, 0, "ActionsHandler");
     qmlRegisterType<ChatDocumentHandler>("org.kde.neochat", 1, 0, "ChatDocumentHandler");
+    qmlRegisterType<SpellcheckHighlighter>("org.kde.neochat", 1, 0, "SpellcheckHighlighter");
     qmlRegisterType<RoomListModel>("org.kde.neochat", 1, 0, "RoomListModel");
+    qmlRegisterType<KWebShortcutModel>("org.kde.neochat", 1, 0, "WebShortcutModel");
     qmlRegisterType<UserListModel>("org.kde.neochat", 1, 0, "UserListModel");
+    qmlRegisterType<CustomEmojiModel>("org.kde.neochat", 1, 0, "CustomEmojiModel");
     qmlRegisterType<MessageEventModel>("org.kde.neochat", 1, 0, "MessageEventModel");
     qmlRegisterType<MessageFilterModel>("org.kde.neochat", 1, 0, "MessageFilterModel");
     qmlRegisterType<PublicRoomListModel>("org.kde.neochat", 1, 0, "PublicRoomListModel");
     qmlRegisterType<UserDirectoryListModel>("org.kde.neochat", 1, 0, "UserDirectoryListModel");
-    qmlRegisterSingletonType<EmojiModel>("org.kde.neochat", 1, 0, "EmojiModel", [](QQmlEngine *engine2, QJSEngine *scriptEngine) -> QObject * {
-        Q_UNUSED(scriptEngine);
-        Q_UNUSED(engine2);
-        return new EmojiModel();
-    });
-    qmlRegisterSingletonType<CommandModel>("org.kde.neochat", 1, 0, "CommandModel", [](QQmlEngine *engine2, QJSEngine *scriptEngine) -> QObject * {
-        Q_UNUSED(scriptEngine);
-        Q_UNUSED(engine2);
-        return new CommandModel();
-    });
     qmlRegisterType<SortFilterRoomListModel>("org.kde.neochat", 1, 0, "SortFilterRoomListModel");
     qmlRegisterType<DevicesModel>("org.kde.neochat", 1, 0, "DevicesModel");
     qmlRegisterUncreatableType<RoomMessageEvent>("org.kde.neochat", 1, 0, "RoomMessageEvent", "ENUM");
-    qmlRegisterUncreatableType<RoomType>("org.kde.neochat", 1, 0, "RoomType", "ENUM");
+    qmlRegisterUncreatableType<NeoChatRoomType>("org.kde.neochat", 1, 0, "NeoChatRoomType", "ENUM");
     qmlRegisterUncreatableType<UserType>("org.kde.neochat", 1, 0, "UserType", "ENUM");
 
     qRegisterMetaType<User *>("User*");
@@ -196,6 +228,7 @@ int main(int argc, char *argv[])
 
     engine.addImportPath("qrc:/imports");
     engine.addImageProvider(QLatin1String("mxc"), new MatrixImageProvider);
+    engine.addImageProvider(QLatin1String("blurhash"), new BlurhashImageProvider);
 
     engine.load(QUrl(QStringLiteral("qrc:/qml/main.qml")));
     if (engine.rootObjects().isEmpty()) {
@@ -213,7 +246,7 @@ int main(int argc, char *argv[])
             auto view = qobject_cast<QQuickWindow *>(obj);
             if (view) {
                 view->show();
-                view->raise();
+                raiseWindow(view);
                 return;
             }
         }
