@@ -18,8 +18,8 @@
 #include <QQmlEngine> // for qmlRegisterType()
 #include <QTimeZone>
 
-#include <KLocalizedString>
 #include <KFormat>
+#include <KLocalizedString>
 
 #include "utils.h"
 
@@ -37,6 +37,7 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[HighlightRole] = "isHighlighted";
     roles[SpecialMarksRole] = "marks";
     roles[LongOperationRole] = "progressInfo";
+    roles[FileMimetypeIcon] = "fileMimetypeIcon";
     roles[AnnotationRole] = "annotation";
     roles[EventResolvedTypeRole] = "eventResolvedType";
     roles[ReplyRole] = "reply";
@@ -56,19 +57,18 @@ MessageEventModel::MessageEventModel(QObject *parent)
     using namespace Quotient;
     qmlRegisterAnonymousType<FileTransferInfo>("org.kde.neochat", 1);
     qRegisterMetaType<FileTransferInfo>();
-    qmlRegisterUncreatableType<EventStatus>("org.kde.neochat", 1, 0, "EventStatus", "EventStatus is not an creatable type");
 
-    QTimer::singleShot(0, this, [=]() {
+    QTimer::singleShot(0, this, [this]() {
         if (!m_currentRoom) {
             return;
         }
         m_currentRoom->getPreviousContent(50);
-        connect(this, &QAbstractListModel::rowsInserted, this, [=]() {
+        connect(this, &QAbstractListModel::rowsInserted, this, [this]() {
             if (m_currentRoom->readMarkerEventId().isEmpty()) {
                 return;
             }
             const auto it = m_currentRoom->findInTimeline(m_currentRoom->readMarkerEventId());
-            if (it == m_currentRoom->timelineEdge()) {
+            if (it == m_currentRoom->historyEdge()) {
                 m_currentRoom->getPreviousContent(50);
             }
         });
@@ -98,7 +98,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
         lastReadEventId = room->readMarkerEventId();
 
         using namespace Quotient;
-        connect(m_currentRoom, &Room::aboutToAddNewMessages, this, [=](RoomEventsRange events) {
+        connect(m_currentRoom, &Room::aboutToAddNewMessages, this, [this](RoomEventsRange events) {
             if (NeoChatConfig::self()->showFancyEffects()) {
                 for (auto &event : events) {
                     RoomMessageEvent *message = dynamic_cast<RoomMessageEvent *>(event.get());
@@ -134,17 +134,17 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             }
             beginInsertRows({}, timelineBaseIndex(), timelineBaseIndex() + int(events.size()) - 1);
         });
-        connect(m_currentRoom, &Room::aboutToAddHistoricalMessages, this, [=](RoomEventsRange events) {
+        connect(m_currentRoom, &Room::aboutToAddHistoricalMessages, this, [this](RoomEventsRange events) {
             if (rowCount() > 0) {
                 rowBelowInserted = rowCount() - 1; // See #312
             }
             beginInsertRows({}, rowCount(), rowCount() + int(events.size()) - 1);
         });
-        connect(m_currentRoom, &Room::addedMessages, this, [=](int lowest, int biggest) {
+        connect(m_currentRoom, &Room::addedMessages, this, [this](int lowest, int biggest) {
             endInsertRows();
             if (!m_lastReadEventIndex.isValid()) {
                 // no read marker, so see if we need to create one.
-                moveReadMarker(QString(), m_currentRoom->readMarkerEventId());
+                moveReadMarker(m_currentRoom->readMarkerEventId());
             }
             if (biggest < m_currentRoom->maxTimelineIndex()) {
                 auto rowBelowInserted = m_currentRoom->maxTimelineIndex() - biggest + timelineBaseIndex() - 1;
@@ -166,7 +166,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             movingEvent = true;
             // Reverse i because row 0 is bottommost in the model
             const auto row = timelineBaseIndex() - i - 1;
-            Q_ASSERT(beginMoveRows({}, row, row, {}, timelineBaseIndex()));
+            beginMoveRows({}, row, row, {}, timelineBaseIndex());
         });
         connect(m_currentRoom, &Room::pendingEventMerged, this, [this] {
             if (movingEvent) {
@@ -184,7 +184,10 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             beginRemoveRows({}, i, i);
         });
         connect(m_currentRoom, &Room::pendingEventDiscarded, this, &MessageEventModel::endRemoveRows);
-        connect(m_currentRoom, &Room::readMarkerMoved, this, &MessageEventModel::moveReadMarker);
+        connect(m_currentRoom, &Room::readMarkerMoved, this, [this](const QString &fromEventId, const QString &toEventId) {
+            Q_UNUSED(fromEventId);
+            moveReadMarker(toEventId);
+        });
         connect(m_currentRoom, &Room::replacedEvent, this, [this](const RoomEvent *newEvent) {
             refreshLastUserEvents(refreshEvent(newEvent->id()) - timelineBaseIndex());
         });
@@ -197,8 +200,10 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
         connect(m_currentRoom, &Room::fileTransferProgress, this, &MessageEventModel::refreshEvent);
         connect(m_currentRoom, &Room::fileTransferCompleted, this, &MessageEventModel::refreshEvent);
         connect(m_currentRoom, &Room::fileTransferFailed, this, &MessageEventModel::refreshEvent);
+#ifndef QUOTIENT_07
         connect(m_currentRoom, &Room::fileTransferCancelled, this, &MessageEventModel::refreshEvent);
-        connect(m_currentRoom->connection(), &Connection::ignoredUsersListChanged, this, [=] {
+#endif
+        connect(m_currentRoom->connection(), &Connection::ignoredUsersListChanged, this, [this] {
             beginResetModel();
             endResetModel();
         });
@@ -230,10 +235,10 @@ void MessageEventModel::refreshEventRoles(int row, const QVector<int> &roles)
     Q_EMIT dataChanged(idx, idx, roles);
 }
 
-void MessageEventModel::moveReadMarker(const QString &fromEventId, const QString &toEventId)
+void MessageEventModel::moveReadMarker(const QString &toEventId)
 {
     const auto timelineIt = m_currentRoom->findInTimeline(toEventId);
-    if (timelineIt == m_currentRoom->timelineEdge()) {
+    if (timelineIt == m_currentRoom->historyEdge()) {
         return;
     }
     int newRow = int(timelineIt - m_currentRoom->messageEvents().rbegin()) + timelineBaseIndex();
@@ -278,7 +283,7 @@ int MessageEventModel::refreshEventRoles(const QString &id, const QVector<int> &
         row = int(pendingIt - m_currentRoom->pendingEvents().begin());
     } else {
         const auto timelineIt = m_currentRoom->findInTimeline(id);
-        if (timelineIt == m_currentRoom->timelineEdge()) {
+        if (timelineIt == m_currentRoom->historyEdge()) {
             qWarning() << "Trying to refresh inexistent event:" << id;
             return -1;
         }
@@ -367,7 +372,6 @@ int MessageEventModel::rowCount(const QModelIndex &parent) const
     } else {
         return m_currentRoom->timelineSize();
     }
-
 }
 
 bool MessageEventModel::canFetchMore(const QModelIndex &parent) const
@@ -385,7 +389,6 @@ void MessageEventModel::fetchMore(const QModelIndex &parent)
     }
 }
 
-
 inline QVariantMap userAtEvent(NeoChatUser *user, NeoChatRoom *room, const RoomEvent &evt)
 {
     Q_UNUSED(evt)
@@ -412,22 +415,22 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     bool isPending = row < timelineBaseIndex();
 
     if (m_lastReadEventIndex.row() == row) {
-        switch(role) {
+        switch (role) {
         case EventTypeRole:
             return QStringLiteral("readMarker");
-        case TimeRole:
-            {
-                const QDateTime eventDate = data(index(m_lastReadEventIndex.row() + 1, 0), TimeRole).toDateTime();
-                const KFormat format;
-                return format.formatRelativeDateTime(eventDate, QLocale::ShortFormat);
-            }
+        case TimeRole: {
+            const QDateTime eventDate = data(index(m_lastReadEventIndex.row() + 1, 0), TimeRole).toDateTime();
+            const KFormat format;
+            return format.formatRelativeDateTime(eventDate, QLocale::ShortFormat);
+        }
         case SpecialMarksRole:
             return EventStatus::Hidden;
         }
         return {};
     }
 
-    const auto timelineIt = m_currentRoom->messageEvents().crbegin() + std::max(0, row - timelineBaseIndex() - (m_lastReadEventIndex.isValid() && m_lastReadEventIndex.row() < row ? 1 : 0));
+    const auto timelineIt = m_currentRoom->messageEvents().crbegin()
+        + std::max(0, row - timelineBaseIndex() - (m_lastReadEventIndex.isValid() && m_lastReadEventIndex.row() < row ? 1 : 0));
     const auto pendingIt = m_currentRoom->pendingEvents().crbegin() + std::min(row, timelineBaseIndex());
     const auto &evt = isPending ? **pendingIt : **timelineIt;
 
@@ -435,7 +438,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         if (evt.isRedacted()) {
             auto reason = evt.redactedBecause()->reason();
             return (reason.isEmpty()) ? i18n("<i>[This message was deleted]</i>")
-                                      : i18n("<i>[This message was deleted: %1]</i>").arg(evt.redactedBecause()->reason());
+                                      : i18n("<i>[This message was deleted: %1]</i>", evt.redactedBecause()->reason());
         }
 
         return m_currentRoom->eventToString(evt, Qt::RichText);
@@ -487,6 +490,9 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         if (evt.isStateEvent()) {
             return "state";
         }
+        if (is<const EncryptedEvent>(evt)) {
+            return "encrypted";
+        }
 
         return "other";
     }
@@ -530,6 +536,15 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         return m_currentRoom->isEventHighlighted(&evt);
     }
 
+    if (role == FileMimetypeIcon) {
+        auto e = eventCast<const RoomMessageEvent>(&evt);
+        if (!e || !e->hasFileContent()) {
+            return QVariant();
+        }
+
+        return e->content()->fileInfo()->mimeType.iconName();
+    }
+
     if (role == SpecialMarksRole) {
         if (isPending) {
             return pendingIt->deliveryStatus();
@@ -620,7 +635,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
             return {};
         };
         const auto replyIt = m_currentRoom->findInTimeline(replyEventId);
-        if (replyIt == m_currentRoom->timelineEdge()) {
+        if (replyIt == m_currentRoom->historyEdge()) {
             return {};
         };
         const auto &replyEvt = **replyIt;
@@ -669,12 +684,11 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
             content = QVariant::fromValue(e->image().originalJson);
         }
 
-        return QVariantMap{
-            {"eventId", replyEventId},
-            {"display", m_currentRoom->eventToString(replyEvt, Qt::RichText)},
-            {"content", content},
-            {"type", type},
-            {"author", userAtEvent(static_cast<NeoChatUser *>(m_currentRoom->user(replyEvt.senderId())), m_currentRoom, evt)}};
+        return QVariantMap{{"eventId", replyEventId},
+                           {"display", m_currentRoom->eventToString(replyEvt, Qt::RichText)},
+                           {"content", content},
+                           {"type", type},
+                           {"author", userAtEvent(static_cast<NeoChatUser *>(m_currentRoom->user(replyEvt.senderId())), m_currentRoom, evt)}};
     }
 
     if (role == ShowAuthorRole) {
@@ -743,8 +757,8 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
 int MessageEventModel::eventIDToIndex(const QString &eventID) const
 {
     const auto it = m_currentRoom->findInTimeline(eventID);
-    if (it == m_currentRoom->timelineEdge()) {
-        //qWarning() << "Trying to find inexistent event:" << eventID;
+    if (it == m_currentRoom->historyEdge()) {
+        // qWarning() << "Trying to find inexistent event:" << eventID;
         return -1;
     }
     return it - m_currentRoom->messageEvents().rbegin() + timelineBaseIndex();
