@@ -30,6 +30,8 @@ public:
         UserMarkerRole,
         FormattedBodyRole,
 
+        FileMimetypeIcon,
+
         ReplyRole,
 
         ShowAuthorRole,
@@ -82,7 +84,7 @@ private:
     void refreshLastUserEvents(int baseTimelineRow);
     void refreshEventRoles(int row, const QVector<int> &roles = {});
     int refreshEventRoles(const QString &eventId, const QVector<int> &roles = {});
-    void moveReadMarker(const QString &fromEventId, const QString &toEventId);
+    void moveReadMarker(const QString &toEventId);
 
 Q_SIGNALS:
     void roomChanged();
