@@ -12,10 +12,14 @@
 #include <QTextDocument>
 #include <functional>
 
+#include <qcoro/qcorosignal.h>
+#include <qcoro/task.h>
+
 #include "connection.h"
 #include "csapi/account-data.h"
 #include "csapi/content-repo.h"
 #include "csapi/leaving.h"
+#include "csapi/redaction.h"
 #include "csapi/room_state.h"
 #include "csapi/rooms.h"
 #include "csapi/typing.h"
@@ -39,7 +43,7 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
 {
     connect(this, &NeoChatRoom::notificationCountChanged, this, &NeoChatRoom::countChanged);
     connect(this, &NeoChatRoom::highlightCountChanged, this, &NeoChatRoom::countChanged);
-    connect(this, &Room::fileTransferCompleted, this, [=] {
+    connect(this, &Room::fileTransferCompleted, this, [this] {
         setFileUploadingProgress(0);
         setHasFileUploading(false);
     });
@@ -48,11 +52,27 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
 
     connect(this, &Quotient::Room::eventsHistoryJobChanged, this, &NeoChatRoom::lastActiveTimeChanged);
 
-    connect(this, &Room::joinStateChanged, this, [=](JoinState oldState, JoinState newState) {
+    connect(this, &Room::joinStateChanged, this, [this](JoinState oldState, JoinState newState) {
         if (oldState == JoinState::Invite && newState != JoinState::Invite) {
             Q_EMIT isInviteChanged();
         }
     });
+    connect(this, &Room::displaynameChanged, this, &NeoChatRoom::displayNameChanged);
+
+    connectSingleShot(this, &Room::baseStateLoaded, this, [this]() {
+        if (this->joinState() != JoinState::Invite) {
+            return;
+        }
+        const QString senderId = getCurrentState<RoomMemberEvent>(localUser()->id())->senderId();
+        QImage avatar_image;
+        if (!user(senderId)->avatarUrl(this).isEmpty()) {
+            avatar_image = user(senderId)->avatar(128, this);
+        } else {
+            qWarning() << "using this room's avatar";
+            avatar_image = avatar(128);
+        }
+        NotificationsManager::instance().postInviteNotification(this, htmlSafeDisplayName(), htmlSafeMemberName(senderId), avatar_image);
+    });
 }
 
 void NeoChatRoom::uploadFile(const QUrl &url, const QString &body)
@@ -63,19 +83,19 @@ void NeoChatRoom::uploadFile(const QUrl &url, const QString &body)
 
     QString txnId = postFile(body.isEmpty() ? url.fileName() : body, url, false);
     setHasFileUploading(true);
-    connect(this, &Room::fileTransferCompleted, [=](const QString &id, const QUrl & /*localFile*/, const QUrl & /*mxcUrl*/) {
+    connect(this, &Room::fileTransferCompleted, [this, txnId](const QString &id, const QUrl & /*localFile*/, const QUrl & /*mxcUrl*/) {
         if (id == txnId) {
             setFileUploadingProgress(0);
             setHasFileUploading(false);
         }
     });
-    connect(this, &Room::fileTransferFailed, [=](const QString &id, const QString & /*error*/) {
+    connect(this, &Room::fileTransferFailed, [this, txnId](const QString &id, const QString & /*error*/) {
         if (id == txnId) {
             setFileUploadingProgress(0);
             setHasFileUploading(false);
         }
     });
-    connect(this, &Room::fileTransferProgress, [=](const QString &id, qint64 progress, qint64 total) {
+    connect(this, &Room::fileTransferProgress, [this, txnId](const QString &id, qint64 progress, qint64 total) {
         if (id == txnId) {
             setFileUploadingProgress(int(float(progress) / float(total) * 100));
         }
@@ -147,10 +167,23 @@ const RoomMessageEvent *NeoChatRoom::lastEvent(bool ignoreStateEvent) const
     return nullptr;
 }
 
+bool NeoChatRoom::lastEventIsSpoiler() const
+{
+    if (auto event = lastEvent()) {
+        if (auto e = eventCast<const RoomMessageEvent>(event)) {
+            if (e->hasTextContent() && e->content() && e->mimeType().name() == "text/html") {
+                auto htmlBody = static_cast<const Quotient::EventContent::TextContent *>(e->content())->body;
+                return htmlBody.contains("data-mx-spoiler");
+            }
+        }
+    }
+    return false;
+}
+
 QString NeoChatRoom::lastEventToString() const
 {
     if (auto event = lastEvent()) {
-        return user(event->senderId())->displayname(this) + (event->isStateEvent() ? " " : ": ") + eventToString(*event);
+        return htmlSafeMemberName(event->senderId()) + (event->isStateEvent() ? " " : ": ") + eventToString(*event);
     }
     return QLatin1String("");
 }
@@ -221,12 +254,12 @@ QDateTime NeoChatRoom::lastActiveTime()
 
 int NeoChatRoom::savedTopVisibleIndex() const
 {
-    return firstDisplayedMarker() == timelineEdge() ? 0 : int(firstDisplayedMarker() - messageEvents().rbegin());
+    return firstDisplayedMarker() == historyEdge() ? 0 : int(firstDisplayedMarker() - messageEvents().rbegin());
 }
 
 int NeoChatRoom::savedBottomVisibleIndex() const
 {
-    return lastDisplayedMarker() == timelineEdge() ? 0 : int(lastDisplayedMarker() - messageEvents().rbegin());
+    return lastDisplayedMarker() == historyEdge() ? 0 : int(lastDisplayedMarker() - messageEvents().rbegin());
 }
 
 void NeoChatRoom::saveViewport(int topIndex, int bottomIndex)
@@ -262,15 +295,13 @@ QVariantList NeoChatRoom::getUsers(const QString &keyword) const
     return matchedList;
 }
 
-QVariantMap NeoChatRoom::getUser(const QString& userID) const
+QVariantMap NeoChatRoom::getUser(const QString &userID) const
 {
     NeoChatUser user(userID, connection());
-    return QVariantMap {
-        { QStringLiteral("id"), user.id() },
-        { QStringLiteral("displayName"), user.displayname(this) },
-        { QStringLiteral("avatarMediaId"), user.avatarMediaId(this) },
-        { QStringLiteral("color"), user.color() }
-    };
+    return QVariantMap{{QStringLiteral("id"), user.id()},
+                       {QStringLiteral("displayName"), user.displayname(this)},
+                       {QStringLiteral("avatarMediaId"), user.avatarMediaId(this)},
+                       {QStringLiteral("color"), user.color()}};
 }
 
 QUrl NeoChatRoom::urlToMxcUrl(const QUrl &mxcUrl)
@@ -302,7 +333,7 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
     using namespace Quotient;
     return visit(
         evt,
-        [prettyPrint, removeReply](const RoomMessageEvent &e) {
+        [this, prettyPrint, removeReply](const RoomMessageEvent &e) {
             using namespace MessageEventContent;
 
             // 1. prettyPrint/HTML
@@ -314,6 +345,10 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
                 htmlBody.replace(utils::userPillRegExp, R"(<b class="user-pill">\1</b>)");
                 htmlBody.replace(utils::strikethroughRegExp, "<s>\\1</s>");
 
+                auto url = connection()->homeserver();
+                auto base = url.scheme() + QStringLiteral("://") + url.host() + (url.port() != -1 ? ':' + QString::number(url.port()) : QString());
+                htmlBody.replace(utils::mxcImageRegExp, QStringLiteral(R"(<img \1 src="%1/_matrix/media/r0/download/\2/\3" \4 > )").arg(base));
+
                 return htmlBody;
             }
 
@@ -351,7 +386,7 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
         },
         [this](const RoomMemberEvent &e) {
             // FIXME: Rewind to the name that was at the time of this event
-            auto subjectName = this->user(e.userId())->displayname();
+            auto subjectName = this->htmlSafeMemberName(e.userId());
             // The below code assumes senderName output in AuthorRole
             switch (e.membership()) {
             case MembershipType::Invite:
@@ -379,7 +414,7 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
                 }
                 // Part 2: profile changes of joined members
                 if (e.isRename() && NeoChatConfig::self()->showRename()) {
-                    if (!e.displayName().isEmpty()) {
+                    if (e.displayName().isEmpty()) {
                         text = i18nc("their refers to a singular user", "cleared their display name");
                     } else {
                         text = i18nc("their refers to a singular user", "changed their display name to %1", e.displayName().toHtmlEscaped());
@@ -418,7 +453,7 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
                 return (e.senderId() != e.userId()) ? i18n("banned %1 from the room: %2", subjectName, e.contentJson()["reason"_ls].toString().toHtmlEscaped())
                                                     : i18n("self-banned from the room");
             case MembershipType::Knock:
-                return i18n("knocked");
+                return i18n("requested an invite");
             default:;
             }
             return i18n("made something unknown");
@@ -442,10 +477,14 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
             return e.isUpgrade() ? i18n("upgraded the room to version %1", e.version().isEmpty() ? "1" : e.version().toHtmlEscaped())
                                  : i18n("created the room, version %1", e.version().isEmpty() ? "1" : e.version().toHtmlEscaped());
         },
+        [](const RoomPowerLevelsEvent &) {
+            return i18nc("'power level' means permission level", "changed the power levels for this room");
+        },
         [](const StateEventBase &e) {
-            // A small hack for state events from TWIM bot
-            return e.stateKey() == "twim" ? i18n("updated the database")
-                : e.stateKey().isEmpty()  ? i18n("updated %1 state", e.matrixType())
+            if (e.matrixType() == QLatin1String("m.room.server_acl")) {
+                return i18n("changed the server access control lists for this room");
+            }
+            return e.stateKey().isEmpty() ? i18n("updated %1 state", e.matrixType())
                                           : i18n("updated %1 state for %2", e.matrixType(), e.stateKey().toHtmlEscaped());
         },
         i18n("Unknown event"));
@@ -455,12 +494,16 @@ void NeoChatRoom::changeAvatar(const QUrl &localFile)
 {
     const auto job = connection()->uploadFile(localFile.toLocalFile());
 #ifdef QUOTIENT_07
-    if(isJobPending(job)) {
+    if (isJobPending(job)) {
 #else
     if (isJobRunning(job)) {
 #endif
         connect(job, &BaseJob::success, this, [this, job] {
+#ifdef QUOTIENT_07
+            connection()->callApi<SetRoomStateWithKeyJob>(id(), "m.room.avatar", localUser()->id(), QJsonObject{{"url", job->contentUri().toString()}});
+#else
             connection()->callApi<SetRoomStateWithKeyJob>(id(), "m.room.avatar", localUser()->id(), QJsonObject{{"url", job->contentUri()}});
+#endif
         });
     }
 }
@@ -532,12 +575,14 @@ QString msgTypeToString(MessageEventType msgType)
     }
 }
 
+QString NeoChatRoom::preprocessText(const QString &text)
+{
+    return markdownToHTML(text);
+}
+
 void NeoChatRoom::postMessage(const QString &rawText, const QString &text, MessageEventType type, const QString &replyEventId, const QString &relateToEventId)
 {
-    const auto html = markdownToHTML(text);
-    QString cleanText(text);
-    cleanText.replace(QRegularExpression("\\[(.+)\\]\\(.+\\)"), "\\1");
-    postHtmlMessage(rawText, html, type, replyEventId, relateToEventId);
+    postHtmlMessage(rawText, text, type, replyEventId, relateToEventId);
 }
 
 void NeoChatRoom::postHtmlMessage(const QString &text, const QString &html, MessageEventType type, const QString &replyEventId, const QString &relateToEventId)
@@ -546,7 +591,7 @@ void NeoChatRoom::postHtmlMessage(const QString &text, const QString &html, Mess
     bool isReply = !replyEventId.isEmpty();
     bool isEdit = !relateToEventId.isEmpty();
     const auto replyIt = findInTimeline(replyEventId);
-    if (replyIt == timelineEdge()) {
+    if (replyIt == historyEdge()) {
         isReply = false;
     }
 
@@ -612,7 +657,7 @@ void NeoChatRoom::toggleReaction(const QString &eventId, const QString &reaction
     }
 
     const auto eventIt = findInTimeline(eventId);
-    if (eventIt == timelineEdge()) {
+    if (eventIt == historyEdge()) {
         return;
     }
 
@@ -677,7 +722,7 @@ bool NeoChatRoom::canSendState(const QString &eventType) const
 bool NeoChatRoom::readMarkerLoaded() const
 {
     const auto it = findInTimeline(readMarkerEventId());
-    return it != timelineEdge();
+    return it != historyEdge();
 }
 
 bool NeoChatRoom::isInvite() const
@@ -689,3 +734,41 @@ bool NeoChatRoom::isUserBanned(const QString &user) const
 {
     return getCurrentState<RoomMemberEvent>(user)->membership() == MembershipType::Ban;
 }
+
+QString NeoChatRoom::htmlSafeName() const
+{
+    return name().toHtmlEscaped();
+}
+
+QString NeoChatRoom::htmlSafeDisplayName() const
+{
+    return displayName().toHtmlEscaped();
+}
+
+void NeoChatRoom::deleteMessagesByUser(const QString &user)
+{
+    doDeleteMessagesByUser(user);
+}
+
+QString NeoChatRoom::joinRule() const
+{
+    return getCurrentState<JoinRulesEvent>()->joinRule();
+}
+
+QCoro::Task<void> NeoChatRoom::doDeleteMessagesByUser(const QString &user)
+{
+    QStringList events;
+    for (const auto &event : messageEvents()) {
+        if (event->senderId() == user && !event->isRedacted() && !event.viewAs<RedactionEvent>() && !event->isStateEvent()) {
+            events += event->id();
+        }
+    }
+    for (const auto &e : events) {
+        auto job = connection()->callApi<RedactEventJob>(id(), QUrl::toPercentEncoding(e), connection()->generateTxnId());
+        co_await qCoro(job, &BaseJob::finished);
+        if (job->error() != BaseJob::Success) {
+            qWarning() << "Error: \"" << job->error() << "\" while deleting messages. Aborting";
+            break;
+        }
+    }
+}
