@@ -5,14 +5,8 @@
 
 #include <QObject>
 
-#include "room.h"
 #include "user.h"
 
-namespace Kirigami
-{
-class PlatformTheme;
-}
-
 using namespace Quotient;
 
 class NeoChatUser : public User
