@@ -26,6 +26,8 @@
 #include "notificationsmanager.h"
 #include "roommanager.h"
 
+Q_DECLARE_METATYPE(Quotient::JoinState)
+
 #ifndef Q_OS_ANDROID
 bool useUnityCounter()
 {
@@ -90,7 +92,7 @@ void RoomListModel::setConnection(Connection *connection)
 
     m_connection = connection;
 
-    for (NeoChatRoom *room : qAsConst(m_rooms)) {
+    for (NeoChatRoom *room : std::as_const(m_rooms)) {
         room->disconnect(this);
     }
 
@@ -99,9 +101,9 @@ void RoomListModel::setConnection(Connection *connection)
     connect(connection, &Connection::joinedRoom, this, &RoomListModel::updateRoom);
     connect(connection, &Connection::leftRoom, this, &RoomListModel::updateRoom);
     connect(connection, &Connection::aboutToDeleteRoom, this, &RoomListModel::deleteRoom);
-    connect(connection, &Connection::directChatsListChanged, this, [=](Quotient::DirectChatsMap additions, Quotient::DirectChatsMap removals) {
+    connect(connection, &Connection::directChatsListChanged, this, [this, connection](Quotient::DirectChatsMap additions, Quotient::DirectChatsMap removals) {
         auto refreshRooms = [this, &connection](Quotient::DirectChatsMap rooms) {
-            for (const QString &roomID : qAsConst(rooms)) {
+            for (const QString &roomID : std::as_const(rooms)) {
                 auto room = connection->room(roomID);
                 if (room) {
                     refresh(static_cast<NeoChatRoom *>(room));
@@ -150,29 +152,29 @@ void RoomListModel::doAddRoom(Room *r)
 
 void RoomListModel::connectRoomSignals(NeoChatRoom *room)
 {
-    connect(room, &Room::displaynameChanged, this, [=] {
+    connect(room, &Room::displaynameChanged, this, [this, room] {
         refresh(room);
     });
-    connect(room, &Room::unreadMessagesChanged, this, [=] {
+    connect(room, &Room::unreadMessagesChanged, this, [this, room] {
         refresh(room);
     });
-    connect(room, &Room::notificationCountChanged, this, [=] {
+    connect(room, &Room::notificationCountChanged, this, [this, room] {
         refresh(room);
     });
     connect(room, &Room::avatarChanged, this, [this, room] {
         refresh(room, {AvatarRole});
     });
-    connect(room, &Room::tagsChanged, this, [=] {
+    connect(room, &Room::tagsChanged, this, [this, room] {
         refresh(room);
     });
-    connect(room, &Room::joinStateChanged, this, [=] {
+    connect(room, &Room::joinStateChanged, this, [this, room] {
         refresh(room);
     });
-    connect(room, &Room::addedMessages, this, [=] {
+    connect(room, &Room::addedMessages, this, [this, room] {
         refresh(room, {LastEventRole});
     });
     connect(room, &Room::notificationCountChanged, this, &RoomListModel::handleNotifications);
-    connect(room, &Room::highlightCountChanged, this, [=] {
+    connect(room, &Room::highlightCountChanged, this, [this, room] {
         if (room->highlightCount() == 0) {
             return;
         }
@@ -203,7 +205,7 @@ void RoomListModel::handleNotifications()
     static QStringList oldNotifications;
     auto job = m_connection->callApi<GetNotificationsJob>();
 
-    connect(job, &BaseJob::success, this, [=]() {
+    connect(job, &BaseJob::success, this, [this, job]() {
         const auto notifications = job->jsonData()["notifications"].toArray();
         if (initial) {
             initial = false;
@@ -247,7 +249,7 @@ void RoomListModel::handleNotifications()
 void RoomListModel::refreshNotificationCount()
 {
     int count = 0;
-    for (auto room : qAsConst(m_rooms)) {
+    for (auto room : std::as_const(m_rooms)) {
         count += room->notificationCount();
     }
     if (m_notificationCount == count) {
@@ -289,7 +291,6 @@ void RoomListModel::updateRoom(Room *room, Room *prev)
     } else {
         beginInsertRows(QModelIndex(), m_rooms.count(), m_rooms.count());
         doAddRoom(newRoom);
-        RoomManager::instance().enterRoom(qobject_cast<NeoChatRoom *>(newRoom));
         endInsertRows();
     }
 }
@@ -328,7 +329,7 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
     }
     NeoChatRoom *room = m_rooms.at(index.row());
     if (role == NameRole) {
-        return !room->name().isEmpty() ? room->name() : room->displayName();
+        return !room->name().isEmpty() ? room->htmlSafeName() : room->htmlSafeDisplayName();
     }
     if (role == DisplayNameRole) {
         return room->displayName();
@@ -341,18 +342,26 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
     }
     if (role == CategoryRole) {
         if (room->joinState() == JoinState::Invite) {
-            return RoomType::Invited;
+            return NeoChatRoomType::Invited;
         }
         if (room->isFavourite()) {
-            return RoomType::Favorite;
+            return NeoChatRoomType::Favorite;
         }
         if (room->isLowPriority()) {
-            return RoomType::Deprioritized;
+            return NeoChatRoomType::Deprioritized;
         }
         if (room->isDirectChat()) {
-            return RoomType::Direct;
+            return NeoChatRoomType::Direct;
         }
-        return RoomType::Normal;
+        const RoomCreateEvent *creationEvent = room->creation();
+        QJsonObject contentJson = creationEvent->contentJson();
+        QJsonObject::const_iterator typeIter = contentJson.find("type");
+        if (typeIter != contentJson.end()) {
+            if (typeIter.value().toString() == "m.space") {
+                return NeoChatRoomType::Space;
+            }
+        }
+        return NeoChatRoomType::Normal;
     }
     if (role == UnreadCountRole) {
         return room->unreadCount();
@@ -364,6 +373,9 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
         return room->highlightCount();
     }
     if (role == LastEventRole) {
+        if (room->lastEventIsSpoiler()) {
+            return QString();
+        }
         return room->lastEventToString();
     }
     if (role == LastActiveTimeRole) {
@@ -373,7 +385,7 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
         if (!room->successorId().isEmpty()) {
             return QStringLiteral("upgraded");
         }
-        return toCString(room->joinState());
+        return QVariant::fromValue(room->joinState());
     }
     if (role == CurrentRoomRole) {
         return QVariant::fromValue(room);
@@ -427,11 +439,33 @@ QString RoomListModel::categoryName(int section)
         return i18n("Normal");
     case 5:
         return i18n("Low priority");
+    case 6:
+        return i18n("Spaces");
     default:
         return "Deadbeef";
     }
 }
 
+QString RoomListModel::categoryIconName(int section)
+{
+    switch (section) {
+    case 1:
+        return QStringLiteral("user-invisible");
+    case 2:
+        return QStringLiteral("favorite");
+    case 3:
+        return QStringLiteral("dialog-messages");
+    case 4:
+        return QStringLiteral("group");
+    case 5:
+        return QStringLiteral("object-order-lower");
+    case 6:
+        return QStringLiteral("group");
+    default:
+        return QStringLiteral("tools-report-bug");
+    }
+}
+
 void RoomListModel::setCategoryVisible(int category, bool visible)
 {
     beginResetModel();
@@ -455,10 +489,15 @@ bool RoomListModel::categoryVisible(int category) const
 
 NeoChatRoom *RoomListModel::roomByAliasOrId(const QString &aliasOrId)
 {
-    for (const auto &room : m_rooms) {
+    for (const auto &room : std::as_const(m_rooms)) {
         if (room->aliases().contains(aliasOrId) || room->id() == aliasOrId) {
             return room;
         }
     }
     return nullptr;
 }
+
+int RoomListModel::indexForRoom(NeoChatRoom *room) const
+{
+    return m_rooms.indexOf(room);
+}
